@@ -1,7 +1,7 @@
 /*
-    Title:      Basic IO.
+    Title:      Basic IO for Unix and similar
 
-    Copyright (c) 2000, 2015-2020 David C. J. Matthews
+    Copyright (c) 2000, 2015-2020, 2022 David C. J. Matthews
 
     Portions of this code are derived from the original stream io
     package copyright CUTS 1983-2000.
@@ -99,6 +99,7 @@ DCJM May 2000.
 #include <stdio.h>
 #endif
 #include <limits>
+#include <climits> // For PATH_MAX
 
 #ifndef INFTIM
 #define INFTIM (-1)
@@ -160,10 +161,10 @@ DCJM May 2000.
 #endif
 
 extern "C" {
-    POLYEXTERNALSYMBOL POLYUNSIGNED PolyChDir(FirstArgument threadId, PolyWord arg);
-    POLYEXTERNALSYMBOL POLYUNSIGNED PolyBasicIOGeneral(FirstArgument threadId, PolyWord code, PolyWord strm, PolyWord arg);
-    POLYEXTERNALSYMBOL POLYUNSIGNED PolyPollIODescriptors(FirstArgument threadId, PolyWord streamVec, PolyWord bitVec, PolyWord maxMillisecs);
-    POLYEXTERNALSYMBOL POLYUNSIGNED PolyPosixCreatePersistentFD(FirstArgument threadId, PolyWord fd);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyChDir(POLYUNSIGNED threadId, POLYUNSIGNED arg);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyBasicIOGeneral(POLYUNSIGNED threadId, POLYUNSIGNED code, POLYUNSIGNED strm, POLYUNSIGNED arg);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyPollIODescriptors(POLYUNSIGNED threadId, POLYUNSIGNED streamVec, POLYUNSIGNED bitVec, POLYUNSIGNED maxMillisecs);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyPosixCreatePersistentFD(POLYUNSIGNED threadId, POLYUNSIGNED fd);
 }
 
 static bool isAvailable(TaskData *taskData, int ioDesc)
@@ -480,18 +481,18 @@ void WaitPoll::Wait(unsigned maxMillisecs)
     if (pollResult < 0) errorResult = ERRORNUMBER;
 }
 
-POLYEXTERNALSYMBOL POLYUNSIGNED PolyPollIODescriptors(FirstArgument threadId, PolyWord streamVector, PolyWord bitVector, PolyWord maxMillisecs)
+POLYEXTERNALSYMBOL POLYUNSIGNED PolyPollIODescriptors(POLYUNSIGNED threadId, POLYUNSIGNED streamVector, POLYUNSIGNED bitVector, POLYUNSIGNED maxMillisecs)
 {
     TaskData *taskData = TaskData::FindTaskForId(threadId);
     ASSERT(taskData != 0);
     taskData->PreRTSCall();
     Handle reset = taskData->saveVec.mark();
-    POLYUNSIGNED maxMilliseconds = maxMillisecs.UnTaggedUnsigned();
+    POLYUNSIGNED maxMilliseconds = PolyWord::FromUnsigned(maxMillisecs).UnTaggedUnsigned();
     Handle result = 0;
 
     try {
-        PolyObject  *strmVec = streamVector.AsObjPtr();
-        PolyObject  *bitVec = bitVector.AsObjPtr();
+        PolyObject  *strmVec = PolyWord::FromUnsigned(streamVector).AsObjPtr();
+        PolyObject  *bitVec = PolyWord::FromUnsigned(bitVector).AsObjPtr();
         POLYUNSIGNED nDesc = strmVec->Length();
         ASSERT(nDesc == bitVec->Length());
 
@@ -612,7 +613,7 @@ static Handle change_dirc(TaskData *taskData, Handle name)
 }
 
 // External call
-POLYUNSIGNED PolyChDir(FirstArgument threadId, PolyWord arg)
+POLYUNSIGNED PolyChDir(POLYUNSIGNED threadId, POLYUNSIGNED arg)
 {
     TaskData *taskData = TaskData::FindTaskForId(threadId);
     ASSERT(taskData != 0);
@@ -646,21 +647,34 @@ Handle isDir(TaskData *taskData, Handle name)
 /* Get absolute canonical path name. */
 Handle fullPath(TaskData *taskData, Handle filename)
 {
-    TempString cFileName;
-
+    TempCString cFileName;
     /* Special case of an empty string. */
     if (PolyStringLength(filename->Word()) == 0) cFileName = strdup(".");
     else cFileName = Poly_string_to_C_alloc(filename->Word());
-    if (cFileName == 0) raise_syscall(taskData, "Insufficient memory", NOMEMORY);
+    if (cFileName == 0) raise_syscall(taskData, "Insufficient memory", ENOMEM);
     TempCString resBuf(realpath(cFileName, NULL));
     if (resBuf == NULL)
-        raise_syscall(taskData, "realpath failed", ERRORNUMBER);
+    {
+#ifdef PATH_MAX
+        if (errno == EINVAL)
+        {
+            // Some very old systems e.g. Solaris 10 don't support NULL for the second arg.
+            resBuf = (char*)malloc(PATH_MAX);
+            if (resBuf == NULL)
+                raise_syscall(taskData, "Insufficient memory", ENOMEM);
+            if (realpath(cFileName, resBuf) == NULL)
+                raise_syscall(taskData, "realpath failed", errno);
+        }
+        else
+#endif
+            raise_syscall(taskData, "realpath failed", errno);
+    }
     /* Some versions of Unix don't check the final component
         of a file.  To be consistent try doing a "stat" of
         the resulting string to check it exists. */
     struct stat fbuff;
     if (stat(resBuf, &fbuff) != 0)
-        raise_syscall(taskData, "stat failed", ERRORNUMBER);
+        raise_syscall(taskData, "stat failed", errno);
     return(SAVE(C_string_to_Poly(taskData, resBuf)));
 }
 
@@ -1063,7 +1077,7 @@ static Handle IO_dispatch_c(TaskData *taskData, Handle args, Handle strm, Handle
 
 // General interface to IO.  Ideally the various cases will be made into
 // separate functions.
-POLYUNSIGNED PolyBasicIOGeneral(FirstArgument threadId, PolyWord code, PolyWord strm, PolyWord arg)
+POLYUNSIGNED PolyBasicIOGeneral(POLYUNSIGNED threadId, POLYUNSIGNED code, POLYUNSIGNED strm, POLYUNSIGNED arg)
 {
     TaskData *taskData = TaskData::FindTaskForId(threadId);
     ASSERT(taskData != 0);
@@ -1089,7 +1103,7 @@ POLYUNSIGNED PolyBasicIOGeneral(FirstArgument threadId, PolyWord code, PolyWord
 }
 
 // Create a persistent file descriptor value for Posix.FileSys.stdin etc.
-POLYEXTERNALSYMBOL POLYUNSIGNED PolyPosixCreatePersistentFD(FirstArgument threadId, PolyWord fd)
+POLYEXTERNALSYMBOL POLYUNSIGNED PolyPosixCreatePersistentFD(POLYUNSIGNED threadId, POLYUNSIGNED fd)
 {
     TaskData *taskData = TaskData::FindTaskForId(threadId);
     ASSERT(taskData != 0);
@@ -1100,7 +1114,7 @@ POLYEXTERNALSYMBOL POLYUNSIGNED PolyPosixCreatePersistentFD(FirstArgument thread
     try {
         result = alloc_and_save(taskData,
             WORDS(SIZEOF_VOIDP), F_BYTE_OBJ | F_MUTABLE_BIT | F_NO_OVERWRITE);
-        *(POLYSIGNED*)(result->Word().AsCodePtr()) = fd.UnTagged() + 1;
+        *(POLYSIGNED*)(result->Word().AsCodePtr()) = PolyWord::FromUnsigned(fd).UnTagged() + 1;
     }
     catch (...) { } // If an ML exception is raised - could have run out of memory
 
