@@ -2,7 +2,7 @@
     Title:  Globals for the system.
     Author:     Dave Matthews, Cambridge University Computer Laboratory
 
-    Copyright David C. J. Matthews 2017-20
+    Copyright David C. J. Matthews 2017-22
 
     Copyright (c) 2000-7
         Cambridge University Technical Services Limited
@@ -329,33 +329,6 @@ public:
 
     bool ContainsNormalLengthWord(void) const { return OBJ_IS_LENGTH(LengthWord()); }
 
-    // Find the start of the constant section for a piece of code.
-    // The first of these is really only needed because we may have objects whose length
-    // words have been overwritten.
-    void GetConstSegmentForCode(POLYUNSIGNED obj_length, PolyWord * &cp, POLYUNSIGNED &count) const
-    {
-        PolyWord *last_word  = Offset(obj_length - 1); // Last word in the code
-        if (last_word->AsSigned() < 0)
-        {
-            cp = last_word + 1 + last_word->AsSigned() / sizeof(PolyWord);
-            count = cp[-1].AsUnsigned();
-        }
-        else
-        {
-            count = last_word->AsUnsigned(); // This is the number of consts
-            cp = last_word - count;
-        }
-    }
-    void GetConstSegmentForCode(PolyWord * &cp, POLYUNSIGNED &count) const
-    {
-        GetConstSegmentForCode(Length(), cp, count);
-    }
-    PolyWord *ConstPtrForCode(void) const
-    {
-        PolyWord *cp; POLYUNSIGNED count;
-        GetConstSegmentForCode(cp, count);
-        return cp;
-    }
     // Follow a chain of forwarding pointers
     PolyObject *FollowForwardingChain(void)
     {
@@ -377,6 +350,7 @@ union stackItem
     POLYCODEPTR codeAddr; // Return addresses
     stackItem* stackAddr; // Stack addresses
     uintptr_t argValue; // Treat an address as an int
+    PolyObject* absAddress; // It could be an absolute address to the heap
 };
 
 
@@ -433,4 +407,21 @@ typedef PolyException poly_exn;
 #define ZERO_X  ""
 #endif
 
+
+// ARM instructions are always little-endian even in big-endian mode
+#ifdef WORDS_BIGENDIAN
+inline uint32_t reverseBytes32(uint32_t value)
+{
+    return (((value & 0x000000ff) << 24) |
+        ((value & 0x0000ff00) << 8) |
+        ((value & 0x00ff0000) >> 8) |
+        ((value & 0xff000000) >> 24));
+}
+inline uint32_t fromARMInstr(uint32_t instr) { return reverseBytes32(instr); }
+inline uint32_t toARMInstr(uint32_t instr) { return reverseBytes32(instr); }
+#else
+inline uint32_t fromARMInstr(uint32_t instr) { return instr; }
+inline uint32_t toARMInstr(uint32_t instr) { return instr; }
+#endif
+
 #endif
